//
// Thanks to Nordic Semiconductor and bjorn-spockeli for important bits and pieces from
// around 2009, (Nordic Case ID 111509).
//

#include <stdbool.h>
#include "XL372.h"
#include "adxl372_spi.h"
#include "SEGGER_RTT.h"
#include "nrf_drv_spi.h"
#include "boards.h"
#include "app_error.h"
#include "app_util_platform.h"
#include "nrf_gpio.h"
#include "nrf_delay.h"
#include <string.h>

static const nrf_drv_spi_t    *p_spi_master_0;


/**@brief Function for initializing the ADXL372 SPI driver.
 *
 * @param[in] nrf_drv_spi_t *m_spi_master    Pointer to spi instance
 *
 */
void ADXL372_SPI_init(const nrf_drv_spi_t *m_spi_master)
{
    uint8_t    value = 0;

    p_spi_master_0 = m_spi_master;
    
    // Since I couldn't get the read register to work I thought maybe you have to
    // turn the adxl372 "on". The data sheet says that upon power up, the device
    // is in Stand-By mode. I thought that maybe a stand-by mode would disallow
    // regester reading. I still don't know. Anyhow, this stuff is my effort to
    // get the device out of stand-by and ready for action, like letting me read
    // a register.
    // Initialize the device per your specifications.
    // Set Data Output Rate
    value = (ODR_400Hz << TIMING_ODR_POS) | (WUR_104ms << TIMING_WUR_POS);
    ADXL372_SPI_writeRegister(XL372_TIMING, &value, 1);
    // Set the Power Control Register
    value = 0;
    value = FULL_BW_MEASUREMENT;
    // Disable High Pass Filter, Low Pass Filter at Bit 3 is enabled with default zero
    value |= 1 << 2;
    ADXL372_SPI_writeRegister(XL372_POWER_CTL, &value, 1); 
    // Set offset values
    value = 0;
    value = BW_200Hz;
    value |= 1 << 3;
    ADXL372_SPI_writeRegister(XL372_MEASURE, &value, 1);
}


/**@brief Function for reading to the ADXL375 registers over SPI.
 *
 * @param[in] address   ADXL372 Register address    
 * @param[in] p_data    Pointer to rx buffer 
 * @param[in] bytes     Number of bytes to be read 
 *
 */
void ADXL372_SPI_readRegister(uint8_t reg_address, uint8_t *p_data, uint8_t bytes)
{
    uint32_t   err_code;
        
    uint8_t    tx_data_spi[bytes];
    memset(tx_data_spi, 0, bytes);

    // Shift the register's address to the left by one, inorder to
    // clear bit 0, and set it high per the adxl372 data sheet's
    // specification for the read command structure of the device's SPI Protocal.
    tx_data_spi[0] = (reg_address << 1) + 1;
    
    // Pull Chip Select line low
    nrf_gpio_pin_clear(SPI0_SS_PIN);
    nrf_delay_ms(100);
    
    err_code = nrf_drv_spi_transfer(p_spi_master_0,
                                    tx_data_spi, bytes,
                                    p_data, bytes);
    
    APP_ERROR_CHECK(err_code);
    
    // Set Chip Select line high
    nrf_delay_ms(100);
    nrf_gpio_pin_set(SPI0_SS_PIN);
}


/**@brief Function for writing to the ADXL372 registers over SPI.
 *
 * @param[in] address   ADXL372 Register address    
 * @param[in] p_data    Pointer to tx buffer 
 * @param[in] bytes     Number of bytes to be written
 *
 */
void ADXL372_SPI_writeRegister(uint8_t reg_address, uint8_t *p_data, uint8_t bytes)
{
         
    uint32_t    err_code;
    
    uint8_t     m_tx_data_spi[bytes + 1];
    memset(m_tx_data_spi, 0, (bytes + 1));      
    memcpy((m_tx_data_spi + 1), p_data, bytes);
    
    // Shift the register's address to the left by one, inorder to
    // clear bit 0, and keep it low per the adxl372 data sheet's
    // specification for the write command structure of the device's SPI Protocal.
    m_tx_data_spi[0] = (reg_address << 1);
    
    // Pull Chip Select line low
    nrf_gpio_pin_clear(SPI0_SS_PIN);
    nrf_delay_ms(10);
     
    err_code = nrf_drv_spi_transfer(p_spi_master_0,
                                    m_tx_data_spi, sizeof(m_tx_data_spi),
                                    NULL, 0);
    
    APP_ERROR_CHECK(err_code);
    
    // Set Chip Select line high
    nrf_delay_ms(10);
    nrf_gpio_pin_set(SPI0_SS_PIN);
}



